﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Clients.Common;
using HeuristicLab.Services.Hive.Common;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using HeuristicLab.Services.Hive.Common.ServiceContracts;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Clients.Hive.Slave.Tests {
  [TestClass]
  public class SlaveTest {

    // Use ClassInitialize to run code before running the first test in the class
    [ClassInitialize]
    public static void MyClassInitialize(TestContext testContext) {
      PluginLoader.pluginAssemblies.Any();
      ServiceLocator.Instance = new MockServiceLocator();
    }

    private List<List<MessageContainer>> CreateMsgsForSingleJob() {
      List<List<MessageContainer>> allMsgs = new List<List<MessageContainer>>();
      //get slave to fetch job
      List<MessageContainer> msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.AquireJob));
      allMsgs.Add(msg);

      //do nothing
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);

      msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.ShutdownSlave));
      allMsgs.Add(msg);
      return allMsgs;
    }

    [TestMethod]
    public void TestSingleJob() {
      Job testJob = new Job { Id = Guid.NewGuid(), JobState = JobState.Waiting, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 0 };
      List<Job> jobList = new List<Job>();
      jobList.Add(testJob);
      MockJob job = new MockJob(400, false);

      using (Disposable<IHiveService> service = ServiceLocator.Instance.GetService()) {
        MockHiveService ms = (MockHiveService)service.Obj;
        ((MockHiveService)service.Obj).Messages = CreateMsgsForSingleJob();
        ((MockHiveService)service.Obj).updateJobs(jobList, job);


        HeuristicLab.Clients.Hive.Salve.Core core = new Salve.Core();
        core.Start();

        Assert.AreEqual<int>(1, ms.ResultJobs.Count);
        Assert.AreEqual<Guid>(testJob.Id, ms.ResultJobs[0].Id);
      }
    }

    private List<List<MessageContainer>> CreateMsgsForShutdownSlaveWhileJobRunning() {
      List<List<MessageContainer>> allMsgs = new List<List<MessageContainer>>();
      //get slave to fetch job
      List<MessageContainer> msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.AquireJob));
      allMsgs.Add(msg);

      msg = new List<MessageContainer>();
      allMsgs.Add(msg);

      msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.ShutdownSlave));
      allMsgs.Add(msg);
      return allMsgs;
    }

    [TestMethod]
    public void TestShutdownSlaveWhileJobRunning() {
      Job testJob = new Job { Id = Guid.NewGuid(), JobState = JobState.Waiting, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 0 };
      List<Job> jobList = new List<Job>();
      jobList.Add(testJob);
      MockJob job = new MockJob(10000, false);

      using (Disposable<IHiveService> service = ServiceLocator.Instance.GetService()) {
        MockHiveService ms = (MockHiveService)service.Obj;
        ((MockHiveService)service.Obj).Messages = CreateMsgsForShutdownSlaveWhileJobRunning();
        ((MockHiveService)service.Obj).updateJobs(jobList, job);


        HeuristicLab.Clients.Hive.Salve.Core core = new Salve.Core();
        core.Start();

      }
    }


    private List<List<MessageContainer>> CreateMsgsForTwoJobs() {
      List<List<MessageContainer>> allMsgs = new List<List<MessageContainer>>();

      //get slave to fetch jobs
      List<MessageContainer> msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.AquireJob));
      allMsgs.Add(msg);

      msg = new List<MessageContainer>();
      allMsgs.Add(msg);

      msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.AquireJob));
      allMsgs.Add(msg);


      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);
      msg = new List<MessageContainer>();
      allMsgs.Add(msg);

      msg = new List<MessageContainer>();
      msg.Add(new MessageContainer(MessageContainer.MessageType.ShutdownSlave));
      allMsgs.Add(msg);
      return allMsgs;
    }

    [TestMethod]
    public void TestTwoJobs() {
      Job testJob1 = new Job { Id = Guid.NewGuid(), JobState = JobState.Waiting, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 0 };
      Job testJob2 = new Job { Id = Guid.NewGuid(), JobState = JobState.Waiting, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 0 };
      List<Job> jobList = new List<Job>();
      jobList.Add(testJob1);
      jobList.Add(testJob2);
      MockJob job = new MockJob(2000, false);

      using (Disposable<IHiveService> service = ServiceLocator.Instance.GetService()) {
        MockHiveService ms = (MockHiveService)service.Obj;
        ((MockHiveService)service.Obj).Messages = CreateMsgsForTwoJobs();
        ((MockHiveService)service.Obj).updateJobs(jobList, job);


        HeuristicLab.Clients.Hive.Salve.Core core = new Salve.Core();
        core.Start();
        Assert.AreEqual<int>(2, ms.ResultJobs.Count);

      }
    }









  }
}
